@@ -5,7 +5,6 @@ import logging
 import os
 import re
 import uuid
-import warnings
 from collections.abc import Mapping, Sequence
 from typing import Any, Literal, NamedTuple, TypedDict
 
@@ -21,6 +20,7 @@ from aind_codeocean_api.models.computations_requests import (
 from typing_extensions import TypeAlias
 
 import npc_lims.exceptions as exceptions
+import npc_lims.paths.s3 as s3
 
 logger = logging.getLogger(__name__)
 
@@ -34,7 +34,7 @@ DataAssetAPI: TypeAlias = dict[
         "last_used",
         "name",
         "size",
-        "sourceBucket",
+        "source_bucket",
         "state",
         "tags",
         "type",
@@ -80,11 +80,11 @@ MODEL_CAPSULE_PIPELINE_MAPPING: dict[str, str] = {
 }
 
 EXAMPLE_JOB_STATUS = {
-    "created": 1708570920,
+    "created": 1710962969,
     "has_results": True,
-    "id": "eadd2f5e-6f3b-4179-8788-5d6e798b1f92",
-    "name": "Run 8570920",
-    "run_time": 92774,
+    "id": "1c900aa5-dde4-475d-bf50-cc96aff9db39",
+    "name": "Run With Parameters 962969",
+    "run_time": 84184,
     "state": "completed",
     "end_status": "succeeded",
 }
@@ -138,12 +138,35 @@ def get_session_data_assets(
     session: str | npc_session.SessionRecord,
 ) -> tuple[DataAssetAPI, ...]:
     session = npc_session.SessionRecord(session)
-    assets = get_subject_data_assets(session.subject)
+    assets: tuple[
+        dict[
+            Literal[
+                "created",
+                "custom_metadata",
+                "description",
+                "files",
+                "id",
+                "last_used",
+                "name",
+                "size",
+                "source_bucket",
+                "state",
+                "tags",
+                "type",
+            ],
+            Any,
+        ],
+        ...,
+    ] = get_subject_data_assets(session.subject)
+    try:
+        pattern = get_codoecean_session_id(session)
+    except ValueError:  # no raw data uploaded
+        pattern = f"ecephys_{session.subject}_{session.date}_{npc_session.PARSE_TIME}"
     return tuple(
         asset
         for asset in assets
         if re.match(
-            f"ecephys_{session.subject}_{session.date}_{npc_session.PARSE_TIME}(_[a-z]*_[a-z]*)*",
+            f"{pattern}(_[a-z]*_[a-z]*)*",
             asset["name"],
         )
     )
@@ -167,50 +190,10 @@ def get_session_result_data_assets(
     return result_data_assets
 
 
-def get_single_data_asset(
-    session: str | npc_session.SessionRecord,
+def get_latest_data_asset(
     data_assets: Sequence[DataAssetAPI],
-    data_asset_type: str,
 ) -> DataAssetAPI:
-    if not data_assets:
-        raise ValueError(
-            f"No {data_asset_type} data assets found for session {session}"
-        )
-
-    session = npc_session.SessionRecord(session)
-
-    if len(data_assets) == 1 and session.idx == 0:
-        return data_assets[0]
-
-    asset_names = tuple(asset["name"] for asset in data_assets)
-    session_times = sorted(
-        {
-            time
-            for time in map(npc_session.extract_isoformat_time, asset_names)
-            if time is not None
-        }
-    )
-    sessions_times_to_assets = {
-        session_time: tuple(
-            asset
-            for asset in data_assets
-            if npc_session.extract_isoformat_time(asset["name"]) == session_time
-        )
-        for session_time in session_times
-    }
-    if 0 < len(session_times) < session.idx + 1:  # 0-indexed
-        raise SessionIndexError(
-            f"Number of assets is less than expected: cannot extract asset for session idx = {session.idx} from {asset_names = }"
-        )
-    data_assets = sessions_times_to_assets[session_times[session.idx]]
-    if len(data_assets) > 1:
-        warnings.warn(
-            f"There is more than one asset for {session = }. Defaulting to most recent: {asset_names}"
-        )
-        created_timestamps = [data_asset["created"] for data_asset in data_assets]
-        most_recent_index = created_timestamps.index(max(created_timestamps))
-        return data_assets[most_recent_index]
-    return data_assets[0]
+    return sorted(data_assets, key=lambda asset: asset["created"])[-1]
 
 
 def get_session_sorted_data_asset(
@@ -218,20 +201,21 @@ def get_session_sorted_data_asset(
 ) -> DataAssetAPI:
     """
     Examples:
-        >>> sorted_data_asset = get_session_sorted_data_asset('668759_20230711')
-        >>> assert isinstance(sorted_data_asset, dict)
+        >>> asset = get_session_sorted_data_asset('ecephys_703333_2024-04-09_1')
+        >>> asset = get_session_sorted_data_asset('668759_20230711')
+        >>> assert isinstance(asset, dict)
     """
     session_result_data_assets = get_session_data_assets(session)
     sorted_data_assets = tuple(
         data_asset
         for data_asset in session_result_data_assets
-        if is_sorted_data_asset(data_asset) and data_asset["files"] > 2
+        if is_sorted_data_asset(data_asset) and data_asset.get("files", -1) > 2
     )
 
     if not sorted_data_assets:
         raise ValueError(f"Session {session} has no sorted data assets")
 
-    return get_single_data_asset(session, sorted_data_assets, "sorted")
+    return get_latest_data_asset(sorted_data_assets)
 
 
 @functools.cache
@@ -310,7 +294,22 @@ def get_session_raw_data_asset(
     if not raw_assets:
         raise ValueError(f"Session {session} has no raw data assets")
 
-    return get_single_data_asset(session, raw_assets, "raw")
+    platforms = tuple(asset["name"].split("_")[0] for asset in raw_assets)
+    if len(set(platforms)) > 1:
+        logger.debug(f"Raw data assets for multiple platforms found for {session}")
+        # if a session has both an ecephys platform raw asset and a behavior platform
+        # asset (which necessarily contains a subset of the ecephys data), we'll take the ecephys asset
+        for platform in ("ecephys", "behavior"):
+            if any(platform in asset["name"] for asset in raw_assets):
+                raw_assets = tuple(
+                    asset for asset in raw_assets if platform in asset["name"]
+                )
+                break
+        else:
+            raise NotImplementedError(
+                f"Raw data assets from multiple platforms found for {session}, which we don't know how to handle: {platforms}"
+            )
+    return get_latest_data_asset(raw_assets)
 
 
 def get_surface_channel_root(session: str | npc_session.SessionRecord) -> upath.UPath:
@@ -326,17 +325,86 @@ def get_surface_channel_root(session: str | npc_session.SessionRecord) -> upath.
         ...
         FileNotFoundError: 649943_20230216 has no surface channel data assets
     """
-    session = npc_session.SessionRecord(session)
-    raw_assets = tuple(
-        asset for asset in get_session_data_assets(session) if is_raw_data_asset(asset)
-    )
+    raw_asset = get_surface_channel_raw_data_asset(session)
+    return get_path_from_data_asset(raw_asset)
+
+
+def get_surface_channel_raw_data_asset(
+    session: str | npc_session.SessionRecord,
+) -> DataAssetAPI:
+    """For a main ephys session (implict idx=0), find a raw asset corresponding to
+    the second session on the same day (idx=1).
+    """
+    session = npc_session.SessionRecord(session).with_idx(1)
     try:
-        raw_asset = get_single_data_asset(session.with_idx(1), raw_assets, "raw")
+        raw_assets = tuple(
+            asset
+            for asset in get_session_data_assets(session)
+            if is_raw_data_asset(asset)
+        )
     except SessionIndexError:
         raise FileNotFoundError(
             f"{session} has no surface channel data assets"
         ) from None
-    return get_path_from_data_asset(raw_asset)
+    if not raw_assets:
+        raise FileNotFoundError(f"{session} has no surface channel data assets")
+    return get_latest_data_asset(raw_assets)
+
+
+@functools.cache
+def get_codoecean_session_id(
+    session: str | npc_session.SessionRecord,
+) -> str:
+    """Get the Code Ocean session ID for a given session, which includes session
+    start time.
+
+    Examples:
+        >>> get_codoecean_session_id('703333_2024-04-09')
+        'ecephys_703333_2024-04-09_13-06-44'
+        >>> get_codoecean_session_id('703333_2024-04-09_1')
+        'ecephys_703333_2024-04-09_15-14-46'
+    """
+    session = npc_session.SessionRecord(session)
+    data_assets = [
+        asset
+        for asset in get_subject_data_assets(session.subject)
+        if (
+            asset["name"].startswith(f"ecephys_{session.subject}_{session.date}")
+            or asset["name"].startswith(f"behavior_{session.subject}_{session.date}")
+        )
+    ]
+
+    asset_names = tuple(asset["name"] for asset in data_assets)
+    session_times = sorted(
+        {
+            time
+            for time in map(npc_session.extract_isoformat_time, asset_names)
+            if time is not None
+        }
+    )
+    session_times_to_assets = {
+        session_time: tuple(
+            asset
+            for asset in data_assets
+            if npc_session.extract_isoformat_time(asset["name"]) == session_time
+        )
+        for session_time in session_times
+    }
+    if not session_times_to_assets:
+        raise ValueError(
+            f"No assets found on codeocean for {session=} - cannot deduce session ID"
+        )
+    if len(session_times) < session.idx + 1:  # 0-indexed
+        raise SessionIndexError(
+            f"Number of assets is less than expected: cannot extract asset for session idx = {session.idx} from {asset_names = }"
+        )
+    session_assets = session_times_to_assets[session_times[session.idx]]
+    session_id = npc_session.extract_aind_session_id(session_assets[0]["name"])
+    assert all(
+        npc_session.extract_aind_session_id(asset["name"]) == session_id
+        for asset in session_assets
+    )
+    return session_id
 
 
 @functools.cache
@@ -355,11 +423,11 @@ def get_raw_data_root(session: str | npc_session.SessionRecord) -> upath.UPath:
 def get_path_from_data_asset(asset: DataAssetAPI) -> upath.UPath:
     """Reconstruct path to raw data in bucket (e.g. on s3) using data asset
     uuid or dict of info from Code Ocean API."""
-    if "sourceBucket" not in asset:
+    if "source_bucket" not in asset:
         raise ValueError(
-            f"Asset {asset['id']} has no `sourceBucket` info - not sure how to create UPath:\n{asset!r}"
+            f"Asset {asset['id']} has no `source_bucket` info - not sure how to create UPath:\n{asset!r}"
         )
-    bucket_info = asset["sourceBucket"]
+    bucket_info = asset["source_bucket"]
     roots = {"aws": "s3", "gcs": "gs"}
     if bucket_info["origin"] not in roots:
         raise RuntimeError(
@@ -394,12 +462,10 @@ def get_session_capsule_pipeline_data_asset(
 ) -> DataAssetAPI:
     """
     Returns the data asset for a given model
+    >>> asset = get_session_capsule_pipeline_data_asset('676909_2023-12-13', 'dlc_eye')
     >>> asset = get_session_capsule_pipeline_data_asset('676909_2023-12-13', 'sorted')
     >>> asset['name']
     'ecephys_676909_2023-12-13_13-43-40_sorted_2024-03-01_16-02-45'
-    >>> asset = get_session_capsule_pipeline_data_asset('676909_2023-12-13', 'dlc_eye')
-    >>> asset['name']
-    'ecephys_676909_2023-12-13_13-43-40_dlc_eye'
     """
     session = npc_session.SessionRecord(session)
 
@@ -410,7 +476,7 @@ def get_session_capsule_pipeline_data_asset(
     if not session_model_asset:
         raise FileNotFoundError(f"{session} has no {process_name} results")
 
-    return get_single_data_asset(session, session_model_asset, process_name)
+    return get_latest_data_asset(session_model_asset)
 
 
 def create_session_data_asset(
@@ -418,7 +484,7 @@ def create_session_data_asset(
 ) -> requests.models.Response | None:
     session = npc_session.SessionRecord(session)
 
-    if is_computation_errorred(computation_id) or not is_computation_finished(
+    if is_computation_errored(computation_id) or not is_computation_finished(
         computation_id
     ):
         return None
@@ -426,15 +492,14 @@ def create_session_data_asset(
     source = aind_codeocean_requests.Source(
         computation=aind_codeocean_requests.Sources.Computation(id=computation_id)
     )
-    tags = ([str(session.subject), "derived", "ephys", "results"],)
-    custom_metadata = (
-        {
-            "data level": "derived data",
-            "experiment type": "ecephys",
-            "modality": "Extracellular electrophysiology",
-            "subject id": str(session.subject),
-        },
-    )
+    tags = [str(session.subject), "derived", "ephys", "results"]
+    custom_metadata = {
+        "data level": "derived data",
+        "experiment type": "ecephys",
+        "modality": "Extracellular electrophysiology",
+        "subject id": str(session.subject),
+    }
+
     create_data_asset_request = aind_codeocean_requests.CreateDataAssetRequest(
         name=data_asset_name,
         mount=data_asset_name,
@@ -462,8 +527,8 @@ def get_job_status(job_id: str, check_files: bool = False) -> CapsuleComputation
     response = get_codeocean_client().get_computation(job_id)
     response.raise_for_status()
     job_status = response.json()
-    if check_files and is_computation_errorred(job_status):
-        logger.info(f"Job {job_status['id']} errorred, updating status")
+    if check_files and is_computation_errored(job_status):
+        logger.info(f"Job {job_status['id']} errored, updating status")
         job_status["end_status"] = "failed"
     return job_status
 
@@ -498,17 +563,22 @@ def get_result_names(job_id: str) -> list[str]:
     return result_item_names
 
 
-def is_computation_errorred(job_id_or_response: str | CapsuleComputationAPI) -> bool:
-    """Job status may say `completed` but the pipeline still errorred: check the
+def is_computation_errored(job_id_or_response: str | CapsuleComputationAPI) -> bool:
+    """Job status may say `completed` but the pipeline still errored: check the
     output folder for indications of error.
 
-    >>> is_computation_errorred(EXAMPLE_JOB_STATUS)
+    - no files (or only `nextflow` and `output` files for pipeline runs)
+    - `end_status` == `failed`
+    - `has_results` == False
+    - `output` file contains `Out of memory.`
+
+    >>> is_computation_errored(EXAMPLE_JOB_STATUS)
     False
-    >>> is_computation_errorred(EXAMPLE_JOB_STATUS | {"end_status": "failed"})
+    >>> is_computation_errored(EXAMPLE_JOB_STATUS | {"end_status": "failed"})
     True
-    >>> is_computation_errorred(EXAMPLE_JOB_STATUS | {"has_results": False})
+    >>> is_computation_errored(EXAMPLE_JOB_STATUS | {"has_results": False})
     True
-    >>> is_computation_errorred(EXAMPLE_JOB_STATUS | {"id": "d5444fc9-9c0f-4c91-90c0-8d17969971b8"})
+    >>> is_computation_errored(EXAMPLE_JOB_STATUS | {"id": "d5444fc9-9c0f-4c91-90c0-8d17969971b8"})
     True
     """
     job_status = _parse_job_id_and_response(job_id_or_response)
@@ -524,7 +594,7 @@ def is_computation_errorred(job_id_or_response: str | CapsuleComputationAPI) ->
         return True
 
     if job_status["state"] == "completed":
-        # check if errorred based on files in result
+        # check if errored based on files in result
         result_item_names = get_result_names(job_id)
         is_no_files = len(result_item_names) == 0
         is_pipeline_error = len(result_item_names) == 2 and result_item_names == [
@@ -539,9 +609,79 @@ def is_computation_errorred(job_id_or_response: str | CapsuleComputationAPI) ->
                 f"Job {job_id} suspected error based on number of files available in result"
             )
             return True
+        if "output" in result_item_names:
+            output = requests.get(
+                get_codeocean_client()
+                .get_result_file_download_url(job_id, "output")
+                .json()["url"]
+            ).text
+            if "Out of memory." in output:
+                logger.debug(
+                    f"Job {job_id} output file includes 'Out of memory.' in text"
+                )
+                return True
+            if "Traceback (most recent call last)" in output:
+                logger.debug(
+                    f"Job {job_id} suspected error based on python traceback in output"
+                )
+                return True
+            if "Command error:" in output:
+                logger.debug(
+                    f"Job {job_id} suspected error based on pipeline error message"
+                )
+                return True
+            if "The CUDA error was:" in output:
+                logger.warning(
+                    f"Job {job_id} suspected failure based on CUDA error message"
+                )
+                # return True - currently (Apr 2024) this results from single
+                # probes not sorting (due to artefacts or other issues), but other
+                # probes still usable
+            if all(
+                text in output.lower()
+                for text in ("sorting", "kilosort", "N E X T F L O W".lower())
+            ):
+                if "nwb" not in result_item_names:
+                    logger.debug(
+                        f"Job {job_id} suspected error based on missing NWB file"
+                    )
+                    return True
     return False
 
 
+def get_skipped_probes(session_id: str | npc_session.SessionRecord) -> str:
+    """Only works with new pipeline output
+
+    Examples:
+        >>> get_skipped_probes('702136_2024-03-05')
+        'E'
+        >>> get_skipped_probes('666986_2023-08-14')
+        'B'
+        >>> get_skipped_probes('668755_2023-08-28')
+        ''
+    """
+    output = get_sorting_output_text(session_id)
+    skipped_probes = ""
+    if "skip" not in output.lower():
+        return skipped_probes
+    for text in output.split("Skipping further processing for this recording")[:-1]:
+        preprocessing = text.split("Preprocessing".upper())[-1]
+        skipped_probes += npc_session.ProbeRecord(preprocessing)
+    return skipped_probes
+
+
+def get_sorting_output_text(session_id: str | npc_session.SessionRecord) -> str:
+    """Contents of the sorting pipeline `output` file (log)"""
+    session = npc_session.SessionRecord(session_id)
+    output_path = next(
+        (p for p in s3.get_sorted_data_paths_from_s3(session) if p.name == "output"),
+        None,
+    )
+    if output_path is None:
+        raise FileNotFoundError(f"No output file found for {session}")
+    return output_path.read_text()
+
+
 if __name__ == "__main__":
     import doctest
 
