@@ -26,6 +26,7 @@ TISSUECYTE_REPO = upath.UPath(
 CODE_OCEAN_DATA_BUCKET = upath.UPath("s3://codeocean-s3datasetsbucket-1u41qdg42ur9")
 
 VIDEO_SUFFIXES = (".mp4", ".avi", ".wmv", ".mov")
+CAMERA_POSITIONS = ("side", "face")
 
 
 def get_data_asset_s3_path(asset_id: str | codeocean.DataAssetAPI) -> upath.UPath:
@@ -37,7 +38,7 @@ def get_data_asset_s3_path(asset_id: str | codeocean.DataAssetAPI) -> upath.UPat
     """
     bucket = CODE_OCEAN_DATA_BUCKET
     with contextlib.suppress(AttributeError, KeyError, TypeError):
-        bucket = upath.UPath(upath.UPath(f's3://{asset_id["sourceBucket"]}'))  # type: ignore[index]
+        bucket = upath.UPath(upath.UPath(f's3://{asset_id["source_bucket"]}'))  # type: ignore[index]
     with contextlib.suppress(AttributeError, KeyError):
         return bucket / asset_id.get("id")  # type: ignore[union-attr, operator]
     with contextlib.suppress(AttributeError):
@@ -63,8 +64,31 @@ def get_raw_data_paths_from_s3(
     first_level_files_directories: Iterator = (
         tuple(directory.iterdir()) for directory in directories
     )
+    first_level_files_directories_unpacked = functools.reduce(
+        operator.add, first_level_files_directories
+    )
 
-    paths = functools.reduce(operator.add, first_level_files_directories)
+    top_level_files_directories: tuple = tuple(
+        file for file in raw_data_root.iterdir() if not file.is_dir()
+    )
+
+    # handle change in upload, where there is now ecephys root folder
+    ephys_file_directories: Iterator = (
+        tuple(directory.iterdir())
+        for directory in first_level_files_directories_unpacked
+        if directory.is_dir() and "ecephys" in directory.stem
+    )
+
+    try:
+        ephys_file_directories_unpacked = functools.reduce(
+            operator.add, ephys_file_directories
+        )
+    except TypeError:  # assume older format
+        ephys_file_directories_unpacked = ()
+
+    paths = first_level_files_directories_unpacked + top_level_files_directories
+    if ephys_file_directories_unpacked:
+        paths = paths + ephys_file_directories_unpacked
 
     if not paths:
         raise FileNotFoundError(
@@ -101,7 +125,7 @@ def get_dlc_eye_s3_paths(
     """
     >>> paths = get_dlc_eye_s3_paths('676909_2023-12-13')
     >>> len(paths)
-    7
+    8
     """
     session = npc_session.SessionRecord(session)
     dlc_eye_data_asset = codeocean.get_session_capsule_pipeline_data_asset(
@@ -128,6 +152,64 @@ def get_dlc_side_s3_paths(
     return tuple(get_data_asset_s3_path(dlc_side_data_asset).iterdir())
 
 
+@functools.cache
+def get_lpfaceparts_s3_dir_paths(
+    session: str | npc_session.SessionRecord,
+) -> tuple[upath.UPath, ...]:
+    """
+    Gets the lighting pose result directories for facial landmark predicitons
+    >>> directories = get_lpfaceparts_s3_dir_paths('702136_2024-03-07')
+    >>> len(directories)
+    4
+    """
+    session = npc_session.SessionRecord(session)
+    lpfaceparts_data_asset = codeocean.get_session_capsule_pipeline_data_asset(
+        session, "LPFaceParts"
+    )
+
+    session_LP_s3_path = get_data_asset_s3_path(lpfaceparts_data_asset)
+    session_LP_s3_directory = tuple(session_LP_s3_path.glob(f"*{session}*"))
+    if not session_LP_s3_directory:
+        raise FileNotFoundError(
+            f"{session} has no lightning pose directory in output. Check codeocean"
+        )
+
+    return tuple(session_LP_s3_directory[0].iterdir())
+
+
+@functools.cache
+def get_lpfaceparts_camera_predictions_s3_paths(
+    session: str | npc_session.SessionRecord, camera: str
+) -> tuple[upath.UPath, ...]:
+    """
+    Gets the lightning pose facial landmark prediction csv paths for the session and camera position
+    >>> side_paths = get_lpfaceparts_camera_predictions_s3_paths('702136_2024-03-07', 'side')
+    >>> len(side_paths)
+    3
+    >>> face_paths = get_lpfaceparts_camera_predictions_s3_paths('702136_2024-03-07', 'face')
+    >>> len(face_paths)
+    3
+    """
+    if camera not in CAMERA_POSITIONS:
+        raise ValueError(f"{camera} is not a currently used camera position")
+
+    session = npc_session.SessionRecord(session)
+    paths = get_lpfaceparts_s3_dir_paths(session)
+    camera_path = tuple(path for path in paths if camera == path.stem)
+    if not camera_path:
+        raise FileNotFoundError(
+            f"{session} has no {camera} lightning pose output. Check codeocean"
+        )
+
+    lp_camera_prediction_paths = tuple(camera_path[0].glob(f"*/*_{session}_*.csv"))
+    if not lp_camera_prediction_paths:
+        raise FileNotFoundError(
+            f"{session} has no {camera} predictions output. Check codeocean"
+        )
+
+    return lp_camera_prediction_paths
+
+
 @functools.cache
 def get_dlc_face_s3_paths(
     session: str | npc_session.SessionRecord,
@@ -205,6 +287,8 @@ def get_recording_dirs_experiment_path_from_s3(
     Examples:
         >>> recording_dirs = get_recording_dirs_experiment_path_from_s3('662892_20230821')
         >>> assert len(recording_dirs) > 0
+        >>> recording_dirs = get_recording_dirs_experiment_path_from_s3('703881_20240509')
+        >>> assert len(recording_dirs) > 0
     """
     raw_data_paths = get_raw_data_paths_from_s3(session)
     recording_dirs = (
@@ -212,6 +296,7 @@ def get_recording_dirs_experiment_path_from_s3(
         for path in raw_data_paths
         if "Record Node" in str(path) and "zarr" not in str(path)
     )
+
     recording_dirs_experiment = tuple(
         next(path.glob("*/recording*")) for path in recording_dirs
     )
@@ -291,7 +376,7 @@ def get_tissuecyte_annotation_files_from_s3(
         >>> electrode_files = get_tissuecyte_annotation_files_from_s3('626791_2022-08-16')
         >>> assert len(electrode_files) > 0
         >>> electrode_files[0].name
-        'Probe_A2_channels_626791_warped_processed.csv'
+        'Probe_A2_channels_626791_warped_processed_new_sorting.csv'
     """
     session = npc_session.SessionRecord(session)
     day = tracked_sessions.get_session_info(session).experiment_day
@@ -304,7 +389,7 @@ def get_tissuecyte_annotation_files_from_s3(
 
     electrode_files = tuple(
         subject_electrode_network_path.glob(
-            f"Probe_*{day}_channels_{str(session.subject.id)}_warped_processed.csv"
+            f"Probe_*{day}_channels_{str(session.subject.id)}_warped_processed_new_sorting.csv"
         )
     )
     if not electrode_files:
