@@ -49,7 +49,7 @@ class SessionInfo:
     experiment_day: int | None = None
     """Experiment day (ephys recording, or opto experiment), starting from 1 for
     each subject. `None` for training behavior-only sessions."""
-    session_kwargs: dict[str, str] = dataclasses.field(default_factory=dict)
+    session_kwargs: dict[str, Any] = dataclasses.field(default_factory=dict)
     notes: str = dataclasses.field(default="")
     issues: list[str] = dataclasses.field(default_factory=list)
 
@@ -92,13 +92,48 @@ class SessionInfo:
         Examples:
             >>> next(session.is_uploaded for session in get_session_info() if session.is_uploaded)
             True
+            >>> get_session_info("behavior_614910_2022-04-04_13-22-02").is_uploaded
+            True
         """
-        if not self.is_ephys:
-            return False  # currently only ephys sessions are uploaded to AIND-codeocean
         with contextlib.suppress(FileNotFoundError, ValueError):
             return bool(codeocean.get_raw_data_root(self.id))
         return False
 
+    @functools.cached_property
+    def raw_data_paths(self) -> tuple[upath.UPath, ...] | None:
+        with contextlib.suppress(FileNotFoundError, ValueError):
+            return s3.get_raw_data_paths_from_s3(self.id)
+        return None
+
+    @functools.cached_property
+    def is_session_json(self) -> bool:
+        if self.raw_data_paths is None:
+            return False
+        return any(p for p in self.raw_data_paths if p.name == "session.json")
+
+    @functools.cached_property
+    def is_rig_json(self) -> bool:
+        if self.raw_data_paths is None:
+            return False
+        return any(p for p in self.raw_data_paths if p.name == "rig.json")
+
+    @functools.cached_property
+    def is_video(self) -> bool:
+        """
+        >>> get_session_info('2022-09-20_13-21-35_628801').is_video
+        False
+        >>> get_session_info('676909_20231212').is_video
+        True
+        """
+        if not self.is_sync:
+            return False
+
+        if (v := self.session_kwargs.get("is_video")) is not None:
+            return v
+
+        # if ephys, we assume there is video
+        return self.is_ephys
+
     @functools.cached_property
     def is_surface_channels(self) -> bool:
         """The session has ephys data collected separately to record surface
@@ -109,6 +144,8 @@ class SessionInfo:
             >>> get_session_info("DRpilot_660023_20230808").is_surface_channels
             True
         """
+        if not self.is_ephys:
+            return False
         if self.session_kwargs.get("probe_letters_with_surface_channel_recording"):
             return True
         try:
@@ -117,59 +154,44 @@ class SessionInfo:
             return False
         return True
 
-    @functools.cached_property
-    def is_dlc_eye(self) -> bool:
-        """
-        The dlc eye capsule has yield a result for this session
-        >>> get_session_info("676909_2023-12-13").is_dlc_eye
-        True
-        """
-
-        if not self.is_ephys:
+    def is_dlc(self, camera: Literal["eye", "side", "face"]) -> bool:
+        if not self.is_video:
             return False
-
         try:
             return bool(
-                codeocean.get_session_capsule_pipeline_data_asset(self.id, "dlc_eye")
+                codeocean.get_session_capsule_pipeline_data_asset(
+                    self.id, f"dlc_{camera}"
+                )
             )
         except (FileNotFoundError, ValueError):
             return False
 
+    @functools.cached_property
+    def is_dlc_eye(self) -> bool:
+        """
+        The dlc eye capsule has yielded a result for this session.
+        >>> get_session_info("676909_2023-12-13").is_dlc_eye
+        True
+        """
+        return self.is_dlc("eye")
+
     @functools.cached_property
     def is_dlc_side(self) -> bool:
         """
-        The dlc side capsule has yield a result for this session
+        The dlc side capsule has yielded a result for this session
         >>> get_session_info("676909_2023-12-13").is_dlc_side
         True
         """
-
-        if not self.is_ephys:
-            return False
-
-        try:
-            return bool(
-                codeocean.get_session_capsule_pipeline_data_asset(self.id, "dlc_side")
-            )
-        except (FileNotFoundError, ValueError):
-            return False
+        return self.is_dlc("side")
 
     @functools.cached_property
     def is_dlc_face(self) -> bool:
         """
-        The dlc face capsule has yield a result for this session
+        The dlc face capsule has yielded a result for this session
         >>> get_session_info("676909_2023-12-13").is_dlc_face
         True
         """
-
-        if not self.is_ephys:
-            return False
-
-        try:
-            return bool(
-                codeocean.get_session_capsule_pipeline_data_asset(self.id, "dlc_face")
-            )
-        except (FileNotFoundError, ValueError):
-            return False
+        return self.is_dlc("face")
 
     @functools.cached_property
     def is_facemap(self) -> bool:
@@ -177,17 +199,43 @@ class SessionInfo:
         The facemap capsule has yield a result for this session
         >>> get_session_info("676909_2023-12-13").is_facemap
         True
+        >>> get_session_info('668759_2023-07-11').is_facemap
+        True
+        >>> get_session_info('706401_2024-04-22').is_facemap
+        True
         """
-
-        if not self.is_ephys:
+        if not self.is_video:
             return False
-
         try:
-            return bool(
-                codeocean.get_session_capsule_pipeline_data_asset(self.id, "facemap")
+            asset = codeocean.get_session_capsule_pipeline_data_asset(
+                self.id, "facemap"
             )
         except (FileNotFoundError, ValueError):
             return False
+        if "83636983-f80d-42d6-a075-09b60c6abd5e" in asset["provenance"]["data_assets"] and self.id != npc_session.SessionRecord("668759_2023-07-11"):  # type: ignore
+            # the capsule had this asset permanently attached for assets made on April 24th/25th. should be resolved
+            # the resulting data are only saved for the wrong asset
+            return False
+        return bool(asset)
+
+    @functools.cached_property
+    def is_LPFaceParts(self) -> bool:
+        """
+        The lightning pose capsule with the facial features has yielded a result for this session
+        >>> get_session_info("702136_2024-03-07").is_LPFaceParts
+        True
+        """
+        if not self.is_video:
+            return False
+
+        try:
+            asset = codeocean.get_session_capsule_pipeline_data_asset(
+                self.id, "LPFaceParts"
+            )
+        except (FileNotFoundError, ValueError):
+            return False
+
+        return bool(asset)
 
     @functools.cached_property
     def is_sorted(self) -> bool:
@@ -206,6 +254,8 @@ class SessionInfo:
                 asset
                 for asset in codeocean.get_session_data_assets(self.id)
                 if "sorted" in asset["name"]
+                and asset["files"]
+                > 6  # number of files produced by sorting pipeline when errorred
             )
         except (FileNotFoundError, ValueError):
             return False
@@ -531,6 +581,7 @@ def _session_info_from_file_contents(contents: FileContents) -> tuple[SessionInf
 
 
 if __name__ == "__main__":
+    get_session_info("behavior_614910_2022-04-04_13-22-02").is_uploaded
     import doctest
 
     import dotenv
